/*
	test program for PROGS SMS C library

	Append a number of files into another  file (first parameter).
	The output file is redirected to the data default. The input files are
	searched on both data and program defaults (in that order).
	If the output file already existed, then the data/files are added at the
	end, otherwise, a new file is created.

	written by Joachim Van der Auwera for PROGS
	(c) 1995 by PROGS, Professional & Graphical Software
	12.07.1996 - a parameter can now also be [-string value], [-line value] or
				 [-var name] to append a string or a "Global Variable"
			   - just return error, don't report
	15.07.1996 - -newline option added, just writes a newline in the file
*/

#include "sms.h"
#include "io.h"
#include "str.h"
#include "thing.h"

#define catch(X) if (err=(X)) return err; else
#define GLOBAL_NAME "Global Variables"
#define GLOBAL_GET	0x47455420	/* "GET " */

Error init(char *param)
{
	Error err;
	Channel out, in;
	char line[256];
	Size len;
	char **list, *base;
	int sent;

	/* get the command line parameters */
	STRBreak(param,&list);
	if (*list==NULL) return ERR_ISYN;

	if (STRSameCI(*list,"-path") && *++list) list++;
	err=IOOpenPath(*list,OPEN_OLD,PATH_DATA,&out);
	if (err) catch( IOOpenPath(*list,OPEN_NEW,PATH_DATA,&out) );
	IOPositionSet(out,TO_END_OF_FILE);
	list++;

	for( ; *list ; list++)
	{
		if (STRSameCI(*list,"-path"))
		{
			if (!*++list) return ERR_IPAR;
		}
		else if (STRSameCI(*list,"-string"))
		{
			if (!*++list) return ERR_IPAR;
			catch( IOPutBytes(out,strlen(*list),*list,&sent) );
		}
		else if (STRSameCI(*list,"-line"))
		{
			if (!*++list) return ERR_IPAR;
			catch( IOPutBytes(out,strlen(*list),*list,&sent) );
			catch( IOPut(out,'\n') );
		}
		else if (STRSameCI(*list,"-newline"))
		{
			catch( IOPut(out,'\n') );
		}
		else if (STRSameCI(*list,"-var"))
		{
			if (!*++list) return ERR_IPAR;
			err=THINGCall(GLOBAL_NAME,GLOBAL_GET,3,*list,line,256);
			if (!err) catch( IOPutBytes(out,strlen(line),line,&sent) );
		}
		else
		{
			catch( IOOpenPath(*list,OPEN_READ,PATH_DATAPROG,&in) );
			catch( IOLength(in,&len) );
			catch( MEMAllocate(len,&base) );
			catch( IOFileLoad(in,len,base) );
			catch( IOPutBytes(out,len,base,&sent) );
			MEMRelease(base);
			IOClose(in);
		}
	}
	return IOClose(out);
}
